/* *********************************************************

class TUtil : Utility class for ROOT
  author    : Eito IWAI ( iwai # champ.hep.sci.osaka-u.ac.jp )

history :
   v0.1.7   add "#define TUtil_h" to avoid redefinitions
            change private to protected
	    add const statement with extractor
   v0.1.6   add TFile::Clear() after Write()
   v0.1.5b  change the treatment of TGraph/TGraphErrors
   v0.1.5a  NewTGraph/NewTGraphErrors are modified ( add skeletonNewTGraph )
            add the certain process to store TGraphs to ROOT file
   v0.1.4   Fix red-line shade around draw area ( Thanks to Nakaya )
            NewTGraph and NewTGraphErrors are modified
   v0.1.3   Draw and Fit are modified ( update skeletonDraw )
            Draw(TTree*) is added
            ViewHists is added
   v0.1.2   Update and CdPad are modified
            SetPadNum,SetPadNumX,SetPadNumY and SetPadNumXY are added
            Draw is modified ( add skeletonDraw )
            remove ClassDef/ClassImp temporary?
   v0.1.1   NewTGraphErrors and Draw(TGraph*) are added
            ClassDef/ClassImp, virtualize destructor are added
   v0.1.0   first version
   
********************************************************** */

#ifndef TUtil_h
#define TUtil_h

#define PSFILE "t.ps"
#define ROOTFILE "t.root"
#define VERSION "TUtil v0.1.7_20080529"

#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <TH1.h>
#include <TCanvas.h>
#include <TPostScript.h>
#include <TFile.h>
#include <TStyle.h>
#include <TROOT.h>
#include <TGraph.h>
#include <TGraphErrors.h>
#include <TTree.h>
#include <TKey.h>
#include <TCollection.h>

class TUtil{
public:
  TUtil(void);
  virtual ~TUtil(void);
  
  // Pad Util
  void CdPad(void);
  void cdPad(void){ CdPad(); }
  void Update(int padNum=1);
  
  // Draw
  void Draw(TH1* hist, char* option="", int logScale=000){ skeletonDraw(hist,option,logScale); }
  void Draw(TH1& hist, char* option="", int logScale=000){ Draw(&hist,option,logScale); }
  void Draw(char* histName, char* option="", int logScale=000){ Draw((TH1*)gROOT->FindObject(histName),option,logScale); }
  void Draw(TH1* hist, int logScale, char* option=""){ Draw(hist,option,logScale); }
  void Draw(TH1& hist, int logScale, char* option=""){ Draw(&hist,option,logScale); }
  void Draw(char* histName, int logScale, char* option=""){ Draw((TH1*)gROOT->FindObject(histName),option,logScale); }
  
  void Draw(TGraph* graph, char* option="", int logScale=000){ skeletonDraw(graph,option,logScale); }
  void Draw(TGraph& graph, char* option="", int logScale=000){ Draw(&graph,option,logScale); }
  
  TH1* Draw(TTree* tree, char* varexp, char* selection="", char* option="", int logScale=000, char* histName="htemp", int nBin=-1, double xlow=0, double xup=0, char* title="<varexp>");
  TH1* Draw(TTree& tree, char* varexp, char* selection="", char* option="", int logScale=000, char* histName="htemp", int nBin=-1, double xlow=0, double xup=0, char* title="<varexp>"){ return Draw(&tree,varexp,selection,option,logScale,histName,nBin,xlow,xup,title); }
  
  // Fit
  void Fit(TH1* hist, char* formula, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ skeletonDraw(hist,goption,logScale,1,formula,option,xmin,xmax); }
  void Fit(TH1& hist, char* formula, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ Fit(&hist,formula,option,logScale,goption,xmin,xmax); }
  void Fit(char* histName, char* formula, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ Fit((TH1*)gROOT->FindObject(histName),formula,option,logScale,goption,xmin,xmax); }
  void Fit(TH1* hist, TF1* f1, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ skeletonDraw(hist,goption,logScale,2,f1,option,xmin,xmax); }
  void Fit(TH1& hist, TF1* f1, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ Fit(&hist,f1,option,logScale,goption,xmin,xmax); }
  void Fit(char* histName, TF1* f1, char* option="", int logScale=000, char* goption="", double xmin=0, double xmax=0){ Fit((TH1*)gROOT->FindObject(histName),f1,option,logScale,goption,xmin,xmax); }
  
  // new blank TGraph ( to be stored to ROOT file )
  TGraph* NewTGraph(char* name="defaultName"){ return skeletonNewTGraph(0,name); }
  TGraphErrors* NewTGraphErrors(char* name="defaultName"){ return (TGraphErrors*)skeletonNewTGraph(1,name); }
  
  // viewer
  void ViewHists(TFile* f, char* searchKey="", TString::ECaseCompare cmp=TString::kIgnoreCase);
  
  // setter
  void SetPadNum(int padNum){ m_padNum=padNum; }
  void SetPadNumX(int padNumX){ SetPadNumXY(padNumX,m_padNumY); }
  void SetPadNumY(int padNumY){ SetPadNumXY(m_padNumX,padNumY); }
  void SetPadNumXY(int padNumX, int padNumY);
  //void SetCanvas(TCanvas& c);
  void SetColorTable(std::vector<int> colorTable){ m_colorTable=colorTable; }
  
  // extractor, getter
  int GetPadNum(void) const { return m_padNum; }
  int GetPadNumX(void) const { return m_padNumX; }
  int GetPadNumY(void) const { return m_padNumY; }
  TCanvas* GetCanvas(void) const { return m_c; }
  std::string GetVersion(void) const { return VERSION; }
  std::string Version(void) const { return GetVersion(); }
  
protected:
  
  // init
  void init_canvas(void);
  void init_ps(void);
  void init_ofile(void);
  void init_style(void);
  void init_colortable(void);
  
  // skeleton
  void skeletonDraw(void* obj, char* option, int logScale, int drawType=0, void* fitFunc=0, char* fitOption="", double xmin=0., double xmax=0.);
  TGraph* skeletonNewTGraph(int graphType, char* name);
  
  int m_padNum;
  int m_padNumX;
  int m_padNumY;
  TCanvas* m_c;
  TPostScript* m_ps;
  TFile* m_f;
  int m_colorTableNum;
  std::vector<int> m_colorTable;
  
  //ClassDef(TUtil,1)
};

//#if !defined(__CINT__)
//ClassImp(TUtil)
//#endif

// constructor
TUtil::TUtil(void):m_padNum(0),m_padNumX(2),m_padNumY(3){
  init_style();
  init_canvas();
  init_ps();
  init_ofile();
  init_colortable();
}

// destructor
TUtil::~TUtil(void){
  m_f->Write();
  m_f->Clear();
  m_ps->Close();
  delete m_f;
  delete m_ps;
  delete m_c;
}

void TUtil::CdPad(void){
  m_c->Update();
  m_padNum++;
  if ( m_padNum==m_padNumX*m_padNumY+1 ){
    Update();
    m_padNum=1;
  }
  m_c->cd(m_padNum);
  m_colorTableNum=0;
}

void TUtil::Update(int padNum){
  if ( !(1<=padNum && padNum<=m_padNumX*m_padNumY) ){
    std::cerr << "[Error] TUtil::Update >> padNum is out of range, ignored." << std::endl;
    return;
  }
  m_c->Update();
  if ( m_padNum!=0 ){ m_ps->NewPage(); }
  for ( int i=0;i<m_padNumX*m_padNumY;i++ ){
    m_c->cd(i+1);
    gPad->SetLogx(0);
    gPad->SetLogy(0);
    gPad->SetLogz(0);
    gPad->Clear();
  }
  m_c->cd(padNum);
  m_padNum=padNum-1;
}

// init
void TUtil::init_canvas(void){
  m_c=new TCanvas("c","ROOT canvas",0,0,640,856);
  m_c->SetFillColor(0);
  m_c->Divide(m_padNumX, m_padNumY, 0.000001, 0.000001);
  
  int i=1;
  while( i<=m_padNumX*m_padNumY ){
    m_c->cd(i);
    gPad->SetGridx();
    gPad->SetGridy();
    i++;
  }
  m_c->cd(1);
}

void TUtil::init_ps(void){
  m_ps=new TPostScript(PSFILE,111);
  m_ps->Range(19,40);
  m_ps->NewPage();
}

void TUtil::init_ofile(void){
  m_f=new TFile(ROOTFILE,"RECREATE");
}

void TUtil::init_colortable(void){
  m_colorTable.push_back(kBlack);
  m_colorTable.push_back(kRed);
  m_colorTable.push_back(kBlue);
  m_colorTable.push_back(kMagenta);
  
  //m_colorTable.push_back(kSpring-1);
  //m_colorTable.push_back(kAzure-3);
  //m_colorTable.push_back(kOrange+1);
}

void TUtil::init_style(void){
  gStyle->SetStatColor(0);
  gStyle->SetTitleFillColor(0);
  gStyle->SetTitleH(0.05);
  gStyle->SetTitleBorderSize(1);
  gStyle->SetStatX(0.84);
  gStyle->SetStatY(0.88);
  gStyle->SetStatFontSize(0.023);
  gStyle->SetStatBorderSize(1);
  gStyle->SetStatStyle(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetLegendBorderSize(1);
  gStyle->SetPadTopMargin(0.12);
  gStyle->SetPadBottomMargin(0.12);
  gStyle->SetPadRightMargin(0.14);
  gStyle->SetPadLeftMargin(0.12);
  gStyle->SetLabelSize(0.05,"X");
  gStyle->SetLabelSize(0.05,"Y");
  gStyle->SetMarkerStyle(20);
  gStyle->SetMarkerSize(0.6);
  gStyle->SetFuncWidth(1);
  gStyle->SetFuncColor(2);
  gStyle->SetEndErrorSize(0);
  gStyle->SetNdivisions(505);
  gStyle->SetPalette(1);
  
  gStyle->SetOptStat("neMri");
  gStyle->SetOptFit(0001);
  gStyle->SetStatW(0.3);
  gStyle->SetStatFontSize(0.03);
}

// Draw
void TUtil::skeletonDraw(void* obj, char* option, int logScale, int drawType, void* fitFunc, char* fitOption, double xmin, double xmax){
  TString t_opt=option;
  bool b_log[3];
  b_log[2]=logScale%2;
  b_log[1]=(logScale%100>2) ? 1 : 0;
  b_log[0]=(logScale>20) ? 1 : 0;
  
  bool b_same=kFALSE;
  TAttLine* t_line=0;
  TAttMarker* t_marker=0;
  TNamed* t_named=0;
  TString t_classname=((TObject*)obj)->ClassName();
  if ( t_classname.Contains("TGraph") ){
    TGraph* drawObj=(TGraph*)obj;
    t_line=drawObj;
    t_marker=drawObj;
    t_named=drawObj;
    if ( !t_opt.Contains("A",TString::kIgnoreCase) ){ b_same=kTRUE; }
  }else if ( t_classname.Contains("TH") ){
    TH1* drawObj=(TH1*)obj;
    t_line=drawObj;
    t_marker=drawObj;
    t_named=drawObj;
    if ( t_opt.Contains("same",TString::kIgnoreCase) ){ b_same=kTRUE; }
  }else if ( t_classname.Contains("TTree") ){
    TTree* drawObj=(TTree*)obj;
    t_line=drawObj;
    t_marker=drawObj;
    t_named=drawObj;
    if ( t_opt.Contains("same",TString::kIgnoreCase) ){ b_same=kTRUE; }
  }else{
    std::cerr << "[Error] TUtil::skeletonDraw >> Unknown object type, ignored." << std::endl;
  }
  
  if ( !b_same ){
    CdPad();
  }else{
    m_colorTableNum++;
    if ( m_colorTableNum>=(int)m_colorTable.size()*2 ){ m_colorTableNum=0; }
    if ( m_colorTableNum<(int)m_colorTable.size() ){
      t_line->SetLineColor(m_colorTable[m_colorTableNum]);
      t_marker->SetMarkerColor(m_colorTable[m_colorTableNum]);
    }else{
      t_line->SetLineStyle(2);
      t_line->SetLineColor(m_colorTable[m_colorTableNum]);
      t_marker->SetMarkerColor(m_colorTable[m_colorTableNum]);
    }
  }
  
  gPad->SetLogx(b_log[0]);
  gPad->SetLogy(b_log[1]);
  gPad->SetLogz(b_log[2]);
  switch(drawType){
  case 0: // Draw
    t_named->Draw(option);
    break;
  case 1: // Fit(char*, ...)
    ((TH1*)obj)->Fit((char*)fitFunc,fitOption,option,xmin,xmax);
    break;
  case 2: // Fit(TF1*, ...)
    ((TH1*)obj)->Fit((TF1*)fitFunc,fitOption,option,xmin,xmax);
    break;
  case 3: // TTree::Draw
    return;
  default:
    std::cerr << "[Error] TUtil::skeletonDraw >> Invarid drawType, ignored." << std::endl;
  }
  m_c->Update();
}

TH1* TUtil::Draw(TTree* tree, char* varexp, char* selection, char* option, int logScale, char* histName, int nBin, double xlow, double xup, char* title){
  std::string t_histname=histName;
  std::string t_title=title;
  bool defaultRange=false;
  bool defaultName=false;
  bool defaultTitle=false;
  if ( nBin==-1 && xlow==0. && xup==0. ){ defaultRange=true; }
  if ( t_histname=="htemp" ){ defaultName=true; }
  if ( t_title=="<varexp>" ){ defaultTitle=true; }
  
  skeletonDraw(tree,option,logScale,3);
  if ( defaultRange ){
    tree->Draw(varexp,selection,option);
  }else{
    std::stringstream sstr;
    sstr << varexp << " >> htemp(" << nBin << "," << xlow << "," << xup << ")";
    tree->Draw(sstr.str().c_str(),selection,option);
  }
  
  TH1* t_hist=(TH1*)(gROOT->FindObject("htemp"));
  if ( !defaultName ){ t_hist->SetName(histName); }
  if ( !defaultTitle ){ t_hist->SetTitle(title); }
  m_c->Update();
  
  return t_hist;
}

TGraph* TUtil::skeletonNewTGraph(int graphType, char* name){
  TGraph* t_gr=NULL;
  switch(graphType){
  case 0: // TGraph
    t_gr=new TGraph;
    break;
  case 1: // TGraphErrors
    t_gr=new TGraphErrors;
    break;
  default:
    std::cerr << "[Error] TUtil::skeletonNewTGraph >> Invarid graphType, ignored." << std::endl;
  }
  
  gDirectory->Append(t_gr);
  if ( name=="defaultName" ){ t_gr->SetName("Graph"); }else{ t_gr->SetName(name); }
  
  return t_gr;
}

void TUtil::SetPadNumXY(int padNumX, int padNumY){
  Update();
  delete m_c;
  m_padNumX=padNumX;
  m_padNumY=padNumY;
  init_canvas();
}

void TUtil::ViewHists(TFile* f, char* searchKey, TString::ECaseCompare cmp){
  TIter t_iter(f->GetListOfKeys());
  TKey* t_key;
  while ( (t_key=(TKey*)t_iter.Next()) ){
    TString t_classname=t_key->GetClassName();
    if ( !t_classname.Contains("TH") ){ continue; }
    TString t_name=t_key->GetName();
    if ( !t_name.Contains(searchKey,cmp) ){ continue; }
    TH1* t_hist=(TH1*)(gROOT->FindObject(t_name));
    Draw(t_hist);
  }
}

#endif // TUtil_h
